package com.jetbyte.cache7;

public abstract class CacheImpl implements Cache
{
   public CacheImpl(String name, String type)
   {
      this.name = name;
      this.type = type;
   }

   protected abstract Object findObject(Cache.Key key);
   protected abstract void addObject(Cache.Key key, Object obj);
   protected abstract void removeObject(Cache.Key key);
   protected abstract void flushCache();

   synchronized public Object find(String keyString)
   {
      key.setKey(keyString);

      return find(key);
   }
   
   synchronized public Object find(Cache.Key key)
   {      
      Object obj = findObject(key);

      if (obj == null)
      {
         misses++;
      }
      else
      {
         hits++;
      }

      return obj;
   }

   public void add(String keyString, Object obj)
   {
      add(new Cache.Key(keyString), obj);
   }

   synchronized public void add(Cache.Key key, Object obj)
   {
      addObject(key, obj);
   }

   synchronized public void remove(String keyString)
   {
      key.setKey(keyString);
      
      remove(key);      
   }

   synchronized public void remove(Cache.Key key)
   {
      removeObject(key);
   }

   public String name()
   {
      return name;
   }

   public String type()
   {
      return type;
   }

   public int hits()
   {
      return hits;
   }

   public int misses()
   {
      return misses;
   }

   synchronized public void flush()
   {
      flushCache();
      
      hits = 0;
      misses = 0;
   }

   private final String name;
   private final String type;

   private int hits = 0;
   private int misses = 0;

   private Cache.Key key = new Cache.Key();   
}
