package com.jetbyte.cache7;

abstract public class TimeLimitedCache extends CacheImpl
{
   public TimeLimitedCache(String name, String type)
   {
      super(name, type);
   }

   abstract protected long getReapPeriod();

   abstract protected void reap();
   
   private ReaperThread reaper = ReaperThread.create(this);

   private static class ReaperThread extends Thread
   {
      private ReaperThread(TimeLimitedCache cache)
      {
         this.cache = cache;      
      }

      public void run()
      {
         // Spread out the reaping for multiple caches with
         // the same reap period by waiting for a random
         // portion of the reap period before starting up...

         pause((long)(Math.random() * cache.getReapPeriod()));

         while (true)
         {
            pause(cache.getReapPeriod());
            
            cache.reap();
         }            
      }

      private void pause(long timeout)
      {
         try
         {
            synchronized(this)
            {
               wait(timeout);
            }
         }
         catch(InterruptedException e)
         {

         }
      }

      private static ReaperThread create(TimeLimitedCache cache)
      {
         ReaperThread reaper = new ReaperThread(cache);

         reaper.setDaemon(true);

         reaper.start();
         
         return reaper;
      }

      private TimeLimitedCache cache;
   }
}
